# Prints all menu nodes that reference a given symbol any of their properties
# or property conditions, along with their parent menu nodes.
#
# Usage:
#
#   $ make [ARCH=<arch>] scriptconfig SCRIPT=Kconfiglib/examples/find_symbol.py SCRIPT_ARG=<name>
#
# Example output for SCRIPT_ARG=X86:
#
#   Found 470 locations that reference X86:
#
#   ========== Location 1 (init/Kconfig:1108) ==========
#
#   config SGETMASK_SYSCALL
#   	bool
#   	prompt "sgetmask/ssetmask syscalls support" if EXPERT
#   	default PARISC || M68K || PPC || MIPS || X86 || SPARC || MICROBLAZE || SUPERH
#   	help
#   	  sys_sgetmask and sys_ssetmask are obsolete system calls
#   	  no longer supported in libc but still enabled by default in some
#   	  architectures.
#
#   	  If unsure, leave the default option here.
#
#   ---------- Parent 1 (init/Kconfig:1077)  ----------
#
#   menuconfig EXPERT
#   	bool
#   	prompt "Configure standard kernel features (expert users)"
#   	select DEBUG_KERNEL
#   	help
#   	  This option allows certain base kernel options and settings
#   	  to be disabled or tweaked. This is for specialized
#   	  environments which can tolerate a "non-standard" kernel.
#   	  Only use this if you really know what you are doing.
#
#   ---------- Parent 2 (init/Kconfig:39)  ----------
#
#   menu "General setup"
#
#   ========== Location 2 (arch/Kconfig:29) ==========
#
#   config OPROFILE_EVENT_MULTIPLEX
#   	bool
#   	prompt "OProfile multiplexing support (EXPERIMENTAL)"
#   	default "n"
#   	depends on OPROFILE && X86
#   	help
#   	  The number of hardware counters is limited. The multiplexing
#   	  feature enables OProfile to gather more events than counters
#   	  are provided by the hardware. This is realized by switching
#   	  between events at a user specified time interval.
#
#   	  If unsure, say N.
#
#   ---------- Parent 1 (arch/Kconfig:16)  ----------
#
#   config OPROFILE
#   	tristate
#   	prompt "OProfile system profiling"
#   	select RING_BUFFER
#   	select RING_BUFFER_ALLOW_SWAP
#   	depends on PROFILING && HAVE_OPROFILE
#   	help
#   	  OProfile is a profiling system capable of profiling the
#   	  whole system, include the kernel, kernel modules, libraries,
#   	  and applications.
#
#   	  If unsure, say N.
#
#   ---------- Parent 2 (init/Kconfig:39)  ----------
#
#   menu "General setup"
#
#   ... (tons more)

import sys

import kconfiglib


if len(sys.argv) < 3:
    sys.exit('Pass symbol name (without "CONFIG_" prefix) with SCRIPT_ARG=<name>')

kconf = kconfiglib.Kconfig(sys.argv[1])
sym_name = sys.argv[2]
if sym_name not in kconf.syms:
    print("No symbol {} exists in the configuration".format(sym_name))
    sys.exit(0)

referencing = [node for node in kconf.node_iter()
               if kconf.syms[sym_name] in node.referenced]
if not referencing:
    print("No references to {} found".format(sym_name))
    sys.exit(0)

print("Found {} locations that reference {}:\n"
      .format(len(referencing), sym_name))

for i, node in enumerate(referencing, 1):
    print("========== Location {} ({}:{}) ==========\n\n{}"
          .format(i, node.filename, node.linenr, node))

    # Print the parents of the menu node too

    node = node.parent
    parent_i = 1
    while node is not kconf.top_node:
        print("---------- Parent {} ({}:{})  ----------\n\n{}"
              .format(parent_i, node.filename, node.linenr, node))
        node = node.parent
        parent_i += 1



#!/usr/bin/env python3

"""
MPFS HSS Generate Profiling Report tool

This script takes the CSV profiling output from the HSS console
which contains function addresses, and tick counts, and converts
the function addresses into function names.

"""

#
#
# MPFS HSS Generate Profiling Report tool
#
# Copyright 2023 Microchip Corporation.
#
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
#
#

import argparse
import csv
import sys

try:
    from elftools.elf.elffile import ELFFile
except ImportError:
    print("Unable to import the pyelftools package, " +
          "please check your Python installation", file=sys.stderr)
    print("Ensure that pyelftools is installed", file=sys.stderr)
    sys.exit(1)


def build_symbol_cache(elf_filepath: str):
    '''takes a path to an ELF file, and builds a symbol cache'''
    if args.verbose:
        print('Building ELF symbol cache from ' + args.elffile,
              file=sys.stderr)

    with open(elf_filepath, 'rb') as f:
        elf = ELFFile(f)

        if args.verbose:
            print('- reading symbols', file=sys.stderr)
        symtab = elf.get_section_by_name('.symtab')

        global symbol_cache
        symbol_cache = {}
        if args.verbose:
            print('- building cache dictionary', file=sys.stderr)
        for symbol in symtab.iter_symbols():
            symbol_cache[symbol['st_value']] = symbol.name


def process_csv(csvfile):
    '''process input CSV'''
    if args.verbose:
        print('Loading sorted CSV', file=sys.stderr)

    with open(csvfile, newline='') as csvfile:
        filtered = (line for line in csvfile if not line.startswith("#"))
        reader = csv.reader(filtered)
        rows = list(reader)
        rows.sort(key=lambda row: int(row[1]), reverse=True)

        if args.verbose:
            print('Searching for function names in symbol cache',
                  file=sys.stderr)

        for row in rows:
            process_row(row)


def process_row(row):
    '''takes a tuple (function address, tick count) and replaces the
    function address with its function name'''
    funcaddr = int(row[0], 16)
    tickcount = int(row[1])

    if funcaddr in symbol_cache:
        print(symbol_cache[funcaddr], tickcount, sep=', ')
    else:
        print(funcaddr, tickcount, sep=', ')
        pass


def main():
    '''main function'''
    parser = argparse.ArgumentParser(description='Generate Profile Report')
    parser.add_argument('--verbose', '-v', action='count', default=0)
    parser.add_argument('elffile')
    parser.add_argument('csvfile')

    global args
    args = parser.parse_args()

    build_symbol_cache(args.elffile)
    process_csv(args.csvfile)


#
#
#

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# Copyright 2019-2022 Microchip FPGA Embedded Systems Solutions.
#
# SPDX-License-Identifier: MIT
#
# MPFS HAL Embedded Software
#
#
#
# This script takes an XML file which describes hardware options and produces
# header files in the target directory which are used by the embedded
# software.
#

import datetime
import os
import os.path
import xml.etree.ElementTree as Etree
import sys

#
# mpfs_configuration_generator.py versions
#
# 0.6.4.1 Cleanup with flake8, ensure that if input XML file specified, we
#       don't subsequently erroneously check the folder for multiple XML
#       files
#
# 0.6.4 Added the following generated files:
#       hw_mssio_mux_alternate.h
#       hw_nvm_map.h
#
# 0.6.3 target folder name change from "fpga_config"
#       to fpga_design config, filename
#       hw_platform.h changed to fpga_design_config.h ,
#       bug fix related to multiple XML file selection and added libero design
#       information constants in fpga_design_config.h/ removed date,version
#       and design information from all the files except fpga_design_config.h
#
# 0.6.2 added support for multiple XML file found in input folder
#       /empty XML file check/ XML filename arg in current folder/
#       if multiple files are there then the file with the latest time stamp
#       will be selected.
#
# 0.6.1 changed target folder name from soc_config to fpga_config
#
# 0.5.2 Aries Embedded Feedback: remove trailing spaces.
#
# 0.5.1 Added check that the source XML document is more recent than content
#       of already existing SoC configuration files.
#
# 0.4.2 Allowed to only specify the folder where the input XML
#       file is located. Use file ending with _mss_cfg.xml if one exists, any
#       other .xml file in the input folder otherwise.
#
# 0.4.1 Modified the arguments to allow specifying
#       the folder where the soc_config folder should be generated.
#
# 0.3.4 fixed comment formatting bug in hw_memory.h generation
#
# 0.3.3 updated copyright format
#
# 0.3.2 removed leading zeros from decimal values (clock rates)
#


def get_script_ver():
    '''
    This changes anytime anytime the mpfs_configuration_generator.py script
    changes. This does not necessarily mean the xml format has been updated in
    get_xml_ver()
    :return: script version
    '''
    return "0.6.4.1"


#
# XML file to parse
# Also an XML files listing tags used for reference
#

reference_xml_file = \
    ('hardware_des_xml,src_example,mpfs_hw_ref_defaults.xml,default',
     'hardware_des_xml,src_example,mpfs_hw_ref_ddr3_100Mhz_ext_clk.xml' +
     ',ddr3_100Mhz_ref')

xml_tag_file = 'hardware_des_xml,src_example,mpfs_hw_tag_reference.xml'

#
# xml tags, the structure here should follow the readme.md description
# contained in the root folder for tags
# Please note: The tag in the first column (mss_xxx) is the same as the
# directory name (/fpga_design_config/mss_xxx)
# the fourth item lets program know how to format info in header file
#         fm_reg - appears as reg with fields
#         fm_define - appears as define with value, no fields
# the six item lets program know how to format value, decimal or hex
#
xml_tags = ('mss_memory_map,map,mem_elements,fm_define,none,hex',
            'mss_memory_map,apb_split,registers,fm_struct,none,hex',
            'mss_memory_map,cache,registers,fm_struct,none,hex',
            'mss_memory_map,pmp_h0,registers,fm_struct,HART0_,hex64',
            'mss_memory_map,pmp_h1,registers,fm_struct,HART1_,hex64',
            'mss_memory_map,pmp_h2,registers,fm_struct,HART2_,hex64',
            'mss_memory_map,pmp_h3,registers,fm_struct,HART3_,hex64',
            'mss_memory_map,pmp_h4,registers,fm_struct,HART4_,hex64',
            'mss_memory_map,mpu_fic0,registers,fm_struct,FIC0_,hex64',
            'mss_memory_map,mpu_fic1,registers,fm_struct,FIC1_,hex64',
            'mss_memory_map,mpu_fic2,registers,fm_struct,FIC2_,hex64',
            'mss_memory_map,mpu_crypto,registers,fm_struct,CRYPTO_,hex64',
            'mss_memory_map,mpu_gem0,registers,fm_struct,GEM0_,hex64',
            'mss_memory_map,mpu_gem1,registers,fm_struct,GEM1_,hex64',
            'mss_memory_map,mpu_usb,registers,fm_struct,USB_,hex64',
            'mss_memory_map,mpu_mmc,registers,fm_struct,MMC_,hex64',
            'mss_memory_map,mpu_scb,registers,fm_struct,SCB_,hex64',
            'mss_memory_map,mpu_trace,registers,fm_struct,TRACE_,hex64',
            'mss_memory_map,nvm_map,registers,fm_define,none,decimal',
            'mss_io,io_mux,registers,fm_reg,none,hex',
            'mss_io,io_mux_alt,registers,fm_reg,none,hex',
            'mss_io,hsio,registers,fm_reg,none,hex',
            'mss_sgmii,tip,registers,fm_reg,none,hex',
            'mss_ddr,options,registers,fm_reg,none,hex',
            'mss_ddr,io_bank,registers,fm_reg,none,hex',
            'mss_ddr,mode,registers,fm_reg,none,hex',
            'mss_ddr,off_mode,registers,fm_reg,none,hex',
            'mss_ddr,segs,registers,fm_reg,none,hex',
            'mss_ddr,ddrc,registers,fm_reg,none,hex',
            'mss_clocks,clocks,registers,fm_define,none,decimal',
            'mss_clocks,mss_sys,registers,fm_define,MSS_,hex',
            'mss_clocks,mss_pll,registers,fm_define,MSS_,hex',
            'mss_clocks,sgmii_pll,registers,fm_reg,SGMII_,hex',
            'mss_clocks,ddr_pll,registers,fm_reg,DDR_,hex',
            'mss_clocks,mss_cfm,registers,fm_reg,MSS_,hex',
            'mss_clocks,sgmii_cfm,registers,fm_reg,SGMII_,hex',
            'mss_general,mss_peripherals,registers,fm_reg,none,hex',)


#
#  Header files to generate
#
header_files = ('fpga_design_config,memory_map,hw_memory.h',
                'fpga_design_config,memory_map,hw_apb_split.h',
                'fpga_design_config,memory_map,hw_cache.h',
                'fpga_design_config,memory_map,hw_pmp_hart0.h',
                'fpga_design_config,memory_map,hw_pmp_hart1.h',
                'fpga_design_config,memory_map,hw_pmp_hart2.h',
                'fpga_design_config,memory_map,hw_pmp_hart3.h',
                'fpga_design_config,memory_map,hw_pmp_hart4.h',
                'fpga_design_config,memory_map,hw_mpu_fic0.h',
                'fpga_design_config,memory_map,hw_mpu_fic1.h',
                'fpga_design_config,memory_map,hw_mpu_fic2.h',
                'fpga_design_config,memory_map,hw_mpu_crypto.h',
                'fpga_design_config,memory_map,hw_mpu_gem0.h',
                'fpga_design_config,memory_map,hw_mpu_gem1.h',
                'fpga_design_config,memory_map,hw_mpu_usb.h',
                'fpga_design_config,memory_map,hw_mpu_mmc.h',
                'fpga_design_config,memory_map,hw_mpu_scb.h',
                'fpga_design_config,memory_map,hw_mpu_trace.h',
                'fpga_design_config,memory_map,hw_nvm_map.h',
                'fpga_design_config,io,hw_mssio_mux.h',
                'fpga_design_config,io,hw_mssio_mux_alternate.h',
                'fpga_design_config,io,hw_hsio_mux.h',
                'fpga_design_config,sgmii,hw_sgmii_tip.h',
                'fpga_design_config,ddr,hw_ddr_options.h',
                'fpga_design_config,ddr,hw_ddr_io_bank.h',
                'fpga_design_config,ddr,hw_ddr_mode.h',
                'fpga_design_config,ddr,hw_ddr_off_mode.h',
                'fpga_design_config,ddr,hw_ddr_segs.h',
                'fpga_design_config,ddr,hw_ddrc.h',
                'fpga_design_config,clocks,hw_mss_clks.h',
                'fpga_design_config,clocks,hw_clk_sysreg.h',
                'fpga_design_config,clocks,hw_clk_mss_pll.h',
                'fpga_design_config,clocks,hw_clk_sgmii_pll.h',
                'fpga_design_config,clocks,hw_clk_ddr_pll.h',
                'fpga_design_config,clocks,hw_clk_mss_cfm.h',
                'fpga_design_config,clocks,hw_clk_sgmii_cfm.h',
                'fpga_design_config,general,hw_gen_peripherals.h')

MAX_LINE_WIDTH = 80


#
# Read the XML file into Etree
#
def read_xml_file(s):
    file_dir = os.path.join(*s)
    tree = Etree.parse(file_dir.strip())
    root = tree.getroot()  # type: object
    return root


#
#  Routine to make a folder
#
def safe_make_folder(i):
    '''Makes a folder (and its parents) if not present'''
    try:
        os.makedirs(i)
    except OSError:
        pass


#
# Create the directory structure
#
def create_hw_dir_struct(root_folder, top):
    '''Creates directory structure off root,' +
    ' subdirectories passed in a tupple'''
    for folder in top:
        safe_make_folder(root_folder + '/' + folder)


#
#  Generate the copyright notice at the top of the header file
#
def write_copyright(root, the_file, filename, creator):
    '''
    generate copyright notice based on the following:
    #/*******************************************************************************
    # * Copyright 2019-yyyy Microchip FPGA Embedded Systems Solutions.
    # *
    # * SPDX-License-Identifier: MIT
    # *
    # * MPFS HAL Embedded Software
    # *
    # */
    :param root:
    :param the_file:
    :param filename:
    :param creator:
    :return:
    '''
    the_file.write('/******************************************************' +
                   '*************************\n')
    the_file.write(" * Copyright 2019-" + str(datetime.datetime.now().year) +
                   " Microchip FPGA Embedded Systems Solutions.\n")
    the_file.write(' *\n')
    the_file.write(' * SPDX-License-Identifier: MIT\n')
    the_file.write(' *\n')
    the_file.write(" * @file " + filename + "\n")
    the_file.write(" * @author " + creator + "\n")
    the_file.write(' *\n')
    if the_file.name == "fpga_design_config\fpga_design_config.h":
        for child in root:
            if child.tag == "design_information":
                for child1 in child:
                    if child1.tag == "design_name":
                        the_file.write(' * Libero design name: ' +
                                       child1.text.strip() + "\n")
                    if child1.tag == "libero_version":
                        the_file.write(' * Generated using Libero version: ' +
                                       child1.text.strip() + "\n")
                    if child1.tag == "mpfs_part_no":
                        the_file.write(' * MPFS part number used in design: ' +
                                       child1.text.strip() + "\n")
                    if child1.tag == "creation_date_time":
                        the_file.write(' * Date generated by Libero: ' +
                                       child1.text.strip() + "\n")
                    if child1.tag == "xml_format_version":
                        the_file.write(' * Format version of XML ' +
                                       'description: ' +
                                       child1.text.strip() + "\n")
        the_file.write(' * PolarFire SoC Configuration Generator version: ' +
                       get_script_ver() + "\n")

    strings = ('',
               ' Note 1: This file should not be edited.' +
               ' If you need to modify a parameter',
               ' without going through regenerating using the MSS' +
               ' Configurator Libero flow ',
               ' or editing the associated XML file',
               ' the following method is recommended: \n',
               ' 1. edit the following file ',
               ' boards/your_board/' +
               'platform_config/mpfs_hal_config/mss_sw_config.h\n',
               ' 2. define the value you want to override there.',
               ' (Note: There is a commented example in the platform' +
               ' directory)\n',
               ' Note 2: The definition in mss_sw_config.h takes' +
               ' precedence, as',
               ' mss_sw_config.h is included prior to the generated header' +
               ' files located in',
               ' boards/your_board/fpga_design_config')
    for string in strings:
        the_file.write(' *' + string + "\n")
    the_file.write(' *\n */\n')


#
#  the header start define
#
def start_define(the_file, filename):
    filename = filename[:-2]  # remove .h from file name
    the_file.write('\n#ifndef ' + filename.upper() + '_H_')
    the_file.write('\n#define ' + filename.upper() + '_H_\n\n')


#
#  start c plus define
#
def start_cplus(the_file, filename):
    the_file.write('\n#ifdef __cplusplus\n')
    the_file.write('extern ' + ' \"C\"' + ' {\n')
    the_file.write('#endif\n\n')


#
#  end define associated with header start define
#
def end_define(the_file, filename):
    filename = filename[:-2]  # remove .h from file name
    the_file.write('\n#endif /*' +
                   ' #ifdef ' + filename.upper() + '_H_ */\n\n')


#
#  end c++ define
#
def end_cplus(the_file, filename):
    the_file.write('\n#ifdef __cplusplus\n}\n#endif\n\n')


#
#  write line, break into chunks
#
def write_line(header_file, reg_description):
    ''' write line, break into chunks '''
    word_list = reg_description.split()  # list of words
    sentence = word_list[0] + ' '
    word_list.pop(0)
    for word in word_list:
        if (len(sentence + word + ' ') > MAX_LINE_WIDTH):
            header_file.write(sentence.rstrip() + '\n')
            sentence = word + ' '
        else:
            sentence = sentence + word + ' '
    if len(sentence) > 0:
        header_file.write(sentence.rstrip() + '\n')


#
# Iterate through registers and produce header file output
#
def generate_register(header_file, registers, tags):
    '''
    Parse registers tag for register tags and print to header file
    :param header_file: header file to print to
    :param registers: registers in a tag
    :param tags: Some tags used to determine print format
    :return:
    '''
    for register in registers:
        # if tag 4 is set, pre-append register name with tag[4] value
        if tags[4] != 'none':
            pre_append = tags[4]
            name = 'LIBERO_SETTING_' + pre_append + register.get('name')
        else:
            name = 'LIBERO_SETTING_' + register.get('name')

        name_of_reg = name
        description = register.get('description')
        name_gap = 15

        if len(name) > 15:
            name_gap = len(name)

        s = '#define' + ' ' + name.ljust(name_gap, ' ')
        name = register.get('name') + "_OFF_MODE"
        name_gap = 15

        if len(name) > 15:
            name_gap = len(name)
        field_list = []
        reg_value = 0

        for field in register:
            if field.tag == "field":
                gap = 30

                if len(field.get('name')) > gap:
                    gap = len(field.get('name')) + 4

                sfield = '    /* ' + field.get('name').ljust(gap, ' ')
                stemp = ('    [' + field.get('offset') + ':' +
                         field.get('width') + ']')
                stemp = stemp.ljust(12, ' ')
                sfield += stemp
                sfield += field.get('Type')

                if (field.get('Type') == 'RW'):
                    sfield += ' value= ' + field.text.strip()
                    temp_val = ((int(field.text.strip(), 16)) <<
                                int(field.get('offset')))
                    reg_value += temp_val

                sfield += ' */\n'
                # add the field to list of fields
                field_list.extend([sfield])

        if tags[5] == 'decimal':
            value = format(reg_value, '01X')
        elif tags[5] == 'hex64':
            value = '0x' + format(reg_value, '016X') + 'ULL'
        else:
            value = '0x' + format(reg_value, '08X') + 'UL'

        name_gap = 4
        if len(s) >= name_gap:
            name_gap = len(s) + 4

        s = s.ljust(name_gap, ' ') + value + '\n'
        reg_description = '/*' + description + ' */\n'
        header_file.write('#if !defined ' + '(' + name_of_reg + ')\n')
        # Write out the register description, max chars per line 80
        write_line(header_file, reg_description)
        header_file.write(s)

        for x in range(len(field_list)):
            header_file.write(field_list[x])

        header_file.write('#endif\n')


#
# Iterate through tag mem_elements looking for mem elements produce header file
# output
#
def generate_mem_elements(header_file, mem_elements, tags):
    '''
    Parse registers tag for mem tags and print to header file
    :param header_file:
    :param registers:
    :return:
    '''
    for mem in mem_elements:
        name = 'LIBERO_SETTING_' + mem.get('name')
        name_of_reg = name
        name_size = name + '_SIZE'
        description = mem.get('description')
        name_gap = 15

        if len(name) > 15:
            name_gap = len(name)

        s = '#define' + ' ' + name.ljust(name_gap, ' ')
        s1 = '#define' + ' ' + name_size.ljust(name_gap, ' ')
        # get the values
        mem_value = mem.text.strip()
        mem_size = mem.get('size')
        # make sure space between name and value 4 spaces
        name_gap = 4

        if len(s) >= name_gap:
            name_gap = len(s) + 4

        # make sure space between name and value 4 spaces
        name_size_gap = 4

        if len(s1) >= name_size_gap:
            name_size_gap = len(s1) + 4

        # create the strings for writing
        s = s.ljust(name_gap, ' ') + mem_value + '\n'
        reg_description = '/*' + description + ' */\n'
        s1 = (s1.ljust(name_size_gap, ' ') + mem_size +
              '    /* Length of memory block*/\n')
        header_file.write('#if !defined ' + '(' + name_of_reg + ')\n')
        header_file.write(reg_description)
        header_file.write(s)
        header_file.write(s1)
        header_file.write('#endif\n')


#
# generate a header file
#
def generate_header(file, real_root, root, file_name, tags):
    creator = "Microchip-FPGA Embedded Systems Solutions"

    with open(file, 'w+') as header_file:
        write_copyright(real_root, header_file, file_name, creator)
        start_define(header_file, file_name)
        start_cplus(header_file, file_name)

        if tags is not None:
            for child in root:
                if child.tag == "registers":
                    generate_register(header_file, child, tags)
                if child.tag == "mem_elements":
                    generate_mem_elements(header_file, child, tags)

                for child2 in child:
                    if child2.tag == "registers":
                        generate_register(header_file, child2, tags)
        else:
            header_file.write('/* No content from MSS Configurator ' +
                              'generated for this file. */\n')
            header_file.write('/* An older version of MSS Configurator has ' +
                              'been used.       */\n')

        end_cplus(header_file, file_name)
        end_define(header_file, file_name)


#
# fpga_design_config.h header file generation.
#

def write_libero_config_info(root, the_file):
    script_version = get_script_ver().split('.')
    tags_dic = {"design_name": "LIBERO_SETTING_DESIGN_NAME",
                "libero_version": "LIBERO_SETTING_MSS_CONFIGURATOR_VERSION",
                "mpfs_part_no": "LIBERO_SETTING_MPFS_PART ",
                "creation_date_time": "LIBERO_SETTING_GENERATION_DATE",
                "xml_format_version": "LIBERO_SETTING_XML_VERSION"}

    # max constant name size + some extra buffer space
    max_gap = max([len(v) for k, v in tags_dic.items()]) + 8

    fixed_gap = 12
    xml_version = []

    for child in root:
        if child.tag == "design_information":
            for child1 in child:
                if child1.tag in tags_dic:
                    gap = max_gap - (len(tags_dic[child1.tag]))
                    the_file.write('#define  ' +
                                   tags_dic[child1.tag].ljust(4, ' ') +
                                   " " * (gap + fixed_gap) + "\"" +
                                   child1.text.strip() + "\"" + "\n")

                    if child1.tag == "xml_format_version":
                        xml_version = child1.text.strip().split('.')

    const = {"LIBERO_SETTING_XML_VERSION_MAJOR": xml_version[0],
             "LIBERO_SETTING_XML_VERSION_MINOR": xml_version[1],
             "LIBERO_SETTING_XML_VERSION_PATCH": xml_version[2],
             "LIBERO_SETTING_HEADER_GENERATOR_VERSION":
             '.'.join(script_version),
             "LIBERO_SETTING_HEADER_GENERATOR_VERSION_MAJOR":
             script_version[0],
             "LIBERO_SETTING_HEADER_GENERATOR_VERSION_MINOR":
             script_version[1],
             "LIBERO_SETTING_HEADER_GENERATOR_VERSION_PATCH":
             script_version[2]}

    # write hard coded constants in the fpga_design_config.h file.
    for k, v in const.items():
        gap = max_gap - len(k)

        if k == "LIBERO_SETTING_HEADER_GENERATOR_VERSION":
            the_file.write('#define  ' + k.ljust(4, ' ') +
                           " " * (gap + fixed_gap) + "\"" + v + "\"" + "\n")
        else:
            the_file.write('#define  ' + k +
                           " " * (gap + fixed_gap) + v + "\n")
    # new line
    the_file.write("\n")


def generate_reference_header_file(ref_header_file, root, header_files):
    creator = "Embedded Software"
    # itemName ="io_mux configuration"
    s = ref_header_file.split(',')
    file = os.path.join(*s)
    file_name = s[-1]
    with open(file, 'w+') as header_file:
        # write the copyright header

        write_copyright(root, header_file, file_name, creator)
        # add exclusive define
        start_define(header_file, file_name)
        # include all the headers

        # define Libero design information constants
        write_libero_config_info(root, header_file)
        index = 0

        for child in header_files:
            c = header_files[index].split(',')
            c.remove('fpga_design_config')
            # include_file = os.path.join(*c)
            # as we need formatting correct for linux and windows
            include_file = c[0] + '/' + c[1]
            header_file.write('#include \"' + include_file + '\"\n')
            index += 1

        # add the c++ define
        start_cplus(header_file, file_name)
        # no content in this case
        comment = ('/* No content in this file, used for ' +
                   'referencing header */\n')
        header_file.write(comment)
        # end the c++ define
        end_cplus(header_file, file_name)
        end_define(header_file, file_name)


#
#  Generate all the header files, passed in output_header_files
#
def generate_header_files(output_header_files, input_xml_file, input_xml_tags):
    '''
    generate a header which references all the generated headers
    '''
    # read in an XML file
    s = input_xml_file.split(',')

    root = read_xml_file(s)
    index = 0

    while index < len(input_xml_tags):
        ref_tags = input_xml_tags[index].split(',')
        s = output_header_files[index].split(',')
        file_name = s[-1]
        dir_name = s[-2]
        file_dir = os.path.join(*s)
        found_match = 0

        for child in root:
            if child.tag == 'mss_' + dir_name:
                for child1 in child:
                    if child1.tag == ref_tags[1]:
                        found_match = 1
                        break
        #
        # Next, create file based on xml content
        #
        if found_match == 1:
            generate_header(file_dir, root, child1, file_name, ref_tags)
        else:
            generate_header(file_dir, root, child1, file_name, None)

        index += 1

    file_name = 'fpga_design_config,fpga_design_config.h'
    generate_reference_header_file(file_name, root, output_header_files)


#
# Return absolute path created from working directory location and relative
# path passed as argument. Handles path to an XML file and path to a folder.
#
def get_full_path(in_path):
    cwd = os.getcwd()
    filename = ''
    temp = in_path

    # if we're given an XML file, use it
    if in_path.endswith('.xml'):
        path_comp = in_path.split('/')
        last = len(path_comp) - 1
        filename = path_comp[last]
        in_path = in_path.replace(filename, '')
    # otherwise, if we have a directory, take the latest timestamped file in it
    else:
        if in_path == '':
            filename = temp
            in_path = os.getcwd()
        else:
            xml_list = []
            dir_entries = os.listdir(in_path)
            for dir_entry in dir_entries:
                if dir_entry.endswith('.xml'):
                    xml_list.append(dir_entry)
                else:
                    if dir_entry.endswith('_mss_cfg.xml'):
                        xml_list.append(dir_entry)
                        break

            # This section will sort the XML file by the latest timestamp
            if len(xml_list) > 1:
                xml_list = sort_by_timestamp(xml_list, in_path)
                filename = xml_list[-1]
                # print("filename is : {}".format(filename))
            else:
                if len(xml_list) != 0:
                    filename = xml_list[0]

    try:
        os.chdir(in_path)
        full_path = os.getcwd()
    except IOError:
        print("caught IO error trying to change directory")
        sys.exit()

    os.chdir(cwd)
    full_path = full_path + '/' + filename

    if is_empty_file(full_path):
        print("\npath is empty")
        sys.exit()
    else:
        return full_path


#
# check is fpath is a file and empty
#
def is_empty_file(fpath):
    return os.path.isfile(fpath) and os.path.getsize(fpath) == 0


#
# sort file names on the basis of time stamp
#
def sort_by_timestamp(file_name, file_path):
    cwd = os.getcwd()
    try:
        os.chdir(file_path)
        path = os.getcwd()
    except IOError:
        print("not a valid folder name--------------")
        sys.exit()

    files = [path + '/' + file_name[i] for i in range(len(file_name))]
    files.sort(key=os.path.getmtime)
    s_file_name = []

    for i in range(len(files)):
        s_file_name.append(files[i].split('/')[-1])

    print("Multiple files found - sorted list: ", s_file_name)
    os.chdir(cwd)
    return s_file_name


#
# helper for showing help information
#
def show_help():
    print('no of args you entered = ' + str(len(sys.argv) - 1))
    print('mpfs_configuration_generator.py :')
    print(' This program reads xml hardware definition, outputs: header files')
    print(' Usage: python3 mpfs_configuration_generator.py ' +
          '[XML file path] [output folder path]')
    print('path can be absolute as well as relative \n')
    input(' Please run again with correct arguments')


#
# main function
# todo: add options from the command line
#
def main_config_generator():
    '''
    This script takes an XML file which describes hardware options and produces
    header files in the target directory which are used by the embedded
    software.
    Currently there are Two command line arguments
    arg0: path to the folder containing XML file.
    arg1: path of the folder where the fpga_design_config will be generated.
    Note - If multiple XML files are present then the one with the latest time
    stamp will be selected.
    '''

    #
    # check/parse  arguments
    #
    nb_arguments = len(sys.argv) - 1

    if nb_arguments < 2:
        show_help()
        sys.exit()

    argument_list = sys.argv[1:]
    input_xml_file = argument_list[0]
    input_xml_file = get_full_path(input_xml_file)
    print("Input XML file: " + input_xml_file)

    if nb_arguments >= 2:
        output_folder_name = argument_list[1]
        output_folder_name = get_full_path(output_folder_name)
        # print("Output folder: " + output_folder_name)
        os.chdir(output_folder_name)

    # Create directory structure for the header files
    #
    root_folder = 'fpga_design_config'
    top = ['clocks', 'ddr', 'io', 'memory_map', 'sgmii', 'general']
    create_hw_dir_struct(root_folder, top)

    #
    # Next, read in XML content and create header files
    #
    generate_header_files(header_files, input_xml_file, xml_tags)
    print('Hardware configuration header files created in directory:',
          os.path.join(output_folder_name, 'fpga_design_config'))


if __name__ == "__main__":
    main_config_generator()



#!/usr/bin/env python3

# Copyright (c) 2019, Ulf Magnusson
# SPDX-License-Identifier: ISC

"""
Saves a minimal configuration file that only lists symbols that differ in value
from their defaults. Loading such a configuration file is equivalent to loading
the "full" configuration file.

Minimal configuration files are handy to start from when editing configuration
files by hand.

The default input configuration file is '.config'. A different input filename
can be passed in the KCONFIG_CONFIG environment variable.

Note: Minimal configurations can also be generated from within the menuconfig
interface.
"""
import argparse

import kconfiglib


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=__doc__)

    parser.add_argument(
        "--kconfig",
        default="Kconfig",
        help="Top-level Kconfig file (default: Kconfig)")

    parser.add_argument(
        "--out",
        metavar="MINIMAL_CONFIGURATION",
        default="defconfig",
        help="Output filename for minimal configuration (default: defconfig)")

    args = parser.parse_args()

    kconf = kconfiglib.Kconfig(args.kconfig, suppress_traceback=True)
    print(kconf.load_config())
    print(kconf.write_min_config(args.out))


if __name__ == "__main__":
    main()



#/*
# *  Copyright (C) 2017 - This file is part of libecc project
# *
# *  Authors:
# *      Ryad BENADJILA <ryadbenadjila@gmail.com>
# *      Arnaud EBALARD <arnaud.ebalard@ssi.gouv.fr>
# *      Jean-Pierre FLORI <jean-pierre.flori@ssi.gouv.fr>
# *
# *  Contributors:
# *      Nicolas VIVET <nicolas.vivet@ssi.gouv.fr>
# *      Karim KHALFALLAH <karim.khalfallah@ssi.gouv.fr>
# *
# *  This software is licensed under a dual BSD and GPL v2 license.
# *  See LICENSE file at the root folder of the project.
# */
#! /usr/bin/env python

import random, sys, re, math, socket, os, select, signal

### Ctrl-C handler
def handler(signal, frame):
    sys.tracebacklimit = 0
    exit(0)

DEFBUFSIZE=32768

def get_cpu_count():
    """
    Try and estimate the number of CPU on the host. First using multiprocessing
    native function, other using content of /proc/cpuinfo. If none of those
    methods did work, 4 is returned.
    """
    try:
        import multiprocessing
        cpucount = multiprocessing.cpu_count()
    except:
        try:
            s = open("/proc/cpuinfo").read()
            cpucount = int(s.split('processor')[-1].split(":")[1].split("\n")[0])
            cpucount += 1
        except:
            cpucount = 4
    return cpucount

# Simple helper to remove comments from a C program
def C_comment_remover(text):
    def replacer(match):
        s = match.group(0)
        if s.startswith('/'):
            return " " # note: a space and not an empty string
        else:
            return s
    regexp = r'//.*?$|/\*.*?\*/|\'(?:\\.|[^\\\'])*\'|"(?:\\.|[^\\"])*"'
    pattern = re.compile(regexp, re.DOTALL | re.MULTILINE)
    return re.sub(pattern, replacer, text)


def egcd(b, n):
    x0, x1, y0, y1 = 1, 0, 0, 1
    while n != 0:
        q, b, n = b // n, n, b % n
        x0, x1 = x1, x0 - q * x1
        y0, y1 = y1, y0 - q * y1
    return  b, x0, y0

def modinv(a, m):
    g, x, y = egcd(a, m)
    if g != 1:
        raise Exception('modular inverse does not exist')
    else:
        return x % m

def is_probprime(n):
    # ensure n is odd
    if n % 2 == 0:
        return False
    # write n-1 as 2**s * d
    # repeatedly try to divide n-1 by 2
    s = 0
    d = n-1
    while True:
        quotient, remainder = divmod(d, 2)
        if remainder == 1:
            break
        s += 1
        d = quotient
    assert(2**s * d == n-1)
    # test the base a to see whether it is a witness for the compositeness of n
    def try_composite(a):
        if pow(a, d, n) == 1:
            return False
        for i in range(s):
            if pow(a, 2**i * d, n) == n-1:
                return False
        return True # n is definitely composite
    for i in range(5):
        a = random.randrange(2, n)
        if try_composite(a):
            return False
    return True # no base tested showed n as composite

def compute_monty_coef(nn_p, p_bitsize):
    """
    Compute montgomery coeff r, r^2 and mpinv. p_bitsize is the size
    of p in bits. It is expected to be a multiple of word
    bit size.
    """
    r = (1 << p_bitsize) % nn_p
    r_square = (1 << (2 * p_bitsize)) % nn_p
    mpinv = 2**wlen - (modinv(nn_p, 2**wlen))
    return r, r_square, mpinv

def compute_div_coef(nn_p, p_bitsize):
    """
    Compute division coeffs p_normalized, p_shift and p_reciprocal.
    """
    tmp = nn_p
    cnt = 0
    while tmp != 0:
        tmp = tmp >> 1
        cnt += 1
    pshift = p_bitsize - cnt
    nn_pnorm = nn_p << pshift
    B = 2**wlen
    prec = B**3 // ((nn_pnorm >> (p_bitsize - 2*wlen)) + 1) - B
    return pshift, nn_pnorm, prec

def getbitlen(bint):
    """
    Returns the number of bits encoding an integer
    """
    if bint == 0:
        return 1
    else:
        return int(bint).bit_length()

def getwlenbitlen(bint, wlen):
    """
    Returns the number of bits encoding an integer
    """
    rounded_wlen_bitlen = ((getbitlen(bint) + wlen - 1) // wlen) * wlen
    if(rounded_wlen_bitlen == 0):
        rounded_wlen_bitlen = wlen
    return rounded_wlen_bitlen


def format_int_string(bint, wlen):
    """
    Returns the string format of an integer rounded to wlen
    """
    rounded_bytelen = (wlen/8) * ((getbitlen(bint) + wlen - 1) // wlen)
    # Special case of zero bit length
    if(rounded_bytelen == 0):
        rounded_bytelen = wlen//8
    return (("%%0%dx" % (2 * rounded_bytelen)) % bint)

def get_random_bigint(wlen, maxwlensize):
    nn_nwords = random.randint(1, maxwlensize)
    nn_maxval = 2 ** (nn_nwords * wlen) - 1
    return random.randint(0, nn_maxval)

if ((len(sys.argv) != 5) and (len(sys.argv) != 6)):
    sys.stderr.write("Usage: %s outfile wlen ntests [tests]\n" % sys.argv[0])
    sys.stderr.write("with\n")
    sys.stderr.write(" outfile: file in which generated tests will be stored\n")
    sys.stderr.write(" wlen   : target architecture word length in bits (64, 32 or 16)\n")
    sys.stderr.write(" maxlen : maximum bit length of tests (e.g. 521 bits, ...)\n")
    sys.stderr.write(" ntests : a multiplier for the number of tests to perform\n")
    sys.stderr.write(" tests  : (optional) specific tests to perform (regexp opcodes) OR\n")
    sys.stderr.write("          a '.c' file where to get the implemented tests. filename\n")
    sys.stderr.write("          is given using e.g. file=fp_pow.c\n")
    sys.exit(-1)

testfile = sys.argv[1]
wlen = int(sys.argv[2])
maxlen = int(sys.argv[3])
ntests = int(sys.argv[4])

nn_logical_tests = ["NN_SHIFT_RIGHT", "NN_SHIFT_LEFT", "NN_ROTATE_RIGHT", "NN_ROTATE_LEFT",
                    "NN_AND", "NN_XOR", "NN_OR", "NN_NOT"]
nn_addition_tests = ["NN_ADD", "NN_SUB", "NN_INC", "NN_DEC", "NN_MOD_ADD", "NN_MOD_SUB", "NN_MOD_INC", "NN_MOD_DEC"]
nn_mul_tests = ["NN_MUL", "NN_MUL_REDC1", "NN_COEF_REDC1", "NN_COEF_DIV" ]
nn_div_tests = ["NN_MOD", "NN_DIVREM", "NN_MODINV", "NN_MODINV_2EXP", "NN_XGCD", "NN_GCD"]

nn_tests = nn_logical_tests + nn_addition_tests + nn_mul_tests + nn_div_tests

fp_add_tests = ["FP_ADD", "FP_SUB"]
fp_mul_tests = ["FP_MUL", "FP_DIV", "FP_MUL_REDC1", "FP_POW"]

fp_tests = fp_add_tests + fp_mul_tests

all_tests = nn_tests + fp_tests

# Get optional specific parameters
asked_tests = all_tests
if (len(sys.argv) == 6):
    # Do we have a .c file given as input?
    if (sys.argv[5])[:5] == "file=":
        file_name = (sys.argv[5])[5:]
        # Open the .c file
        C_string = C_comment_remover(open(file_name, 'r').read())
        # Grep the interesting tests
        lines = C_string.split("\n")
        asked_tests = []
        for line in lines:
            if line[:13] == "GENERIC_TEST(":
                # Get second argument, which is the opcode
                the_test = line.split(",")[1].replace(" ", "")
                asked_tests += [the_test]
    # We have a list of opcode regexps
    else:
        def check_regexp(regexp, string):
            return re.match(regexp+"$", string)
        asked_tests = []
        asked_tests_regexps = ((sys.argv[5]).replace(" ", "")).split(",")
        # Check for regexps
        for regexp in asked_tests_regexps:
            # Asked operations must be known
            match = [x for x in all_tests if check_regexp(regexp, x)]
            if match == []:
                print "Warning: regexp matches no known operation ", regexp
            asked_tests += match

# Unnecessary test (we can keep it though)
if len(list(set(asked_tests) & set(all_tests))) != len(asked_tests):
    print "Error: unknown asked tests ", list(set(asked_tests) - set(all_tests))
    exit(-1)

# Delta to use on word boundaries.
WORD_BOUNDARY_DELTA=3

# Max size (in words) of input numbers (nn, fp) on which to perform tests
MAX_INPUT_PARAM_WLEN= ((maxlen + wlen - 1) // wlen)

test_funcs = { }

# Generate tests for NN_SHIFT_RIGHT and NN_SHIFT_LEFT operations.
def test_NN_SHIFT(op):
    nn_nwords = random.randint(1, MAX_INPUT_PARAM_WLEN)
    nn_maxval = 2 ** (nn_nwords * wlen) - 1
    nn_nbits = nn_nwords * wlen

    nn_val = random.randint(0, nn_maxval)

    res = []

    # try and consider all possible words boundary on input ...
    for boundary in range(0, (nn_nwords + 1) * wlen, wlen):
        # and generate shift for bitcount in an interval around
        # that boundary, i.e. [boundary - delta, boundary + delta]
        min_cnt = max(0, boundary - WORD_BOUNDARY_DELTA)
        max_cnt = min(nn_nbits, boundary + WORD_BOUNDARY_DELTA)
        cnt = random.randint(min_cnt, max_cnt);
        msk = nn_maxval

        # we try different bitlen for output
        for outbitlen in [boundary - wlen, boundary, boundary + wlen]:
            if outbitlen <= wlen:
                outbitlen = wlen

            # Depending on the type of shift operation we consider,
            # we adapt the & mask
            if (op == "NN_SHIFT_RIGHT_FIXEDLEN"): # NN_SHIFT_RIGHT_FIXEDLEN
                msk = (2 ** outbitlen) - 1
                out = (nn_val >> cnt) & msk
            elif (op == "NN_SHIFT_LEFT_FIXEDLEN"): # NN_SHIFT_LEFT_FIXEDLEN
                msk = (2 ** outbitlen) - 1
                out = (nn_val << cnt) & msk
            elif (op == "NN_SHIFT_RIGHT"): # NN_SHIFT_RIGHT
                outbitlen = nn_nbits
                msk = (2 ** outbitlen) - 1
                out = (nn_val >> cnt) & msk
            else: # NN_SHIFT_LEFT
                outbitlen = nn_nbits + cnt
		# Round the bit length to the word boundary
		outbitlen = ((outbitlen + wlen - 1) // wlen) * wlen
                msk = (2 ** outbitlen) - 1
                out = (nn_val << cnt) & msk

            fmt = "%s nnu %s %s %d\n"
            res.append(fmt % (op, format_int_string(out, wlen), format_int_string(nn_val, wlen), cnt))

    return res

test_funcs["NN_SHIFT_RIGHT_FIXEDLEN"] = test_NN_SHIFT
test_funcs["NN_SHIFT_LEFT_FIXEDLEN"]  = test_NN_SHIFT
test_funcs["NN_SHIFT_RIGHT"] = test_NN_SHIFT
test_funcs["NN_SHIFT_LEFT"]  = test_NN_SHIFT

# Generate tests for NN_ROTATE_LEFT and NN_ROTATE_RIGHT operations.
def test_NN_ROTATE(op):
    # random value for both input numbers
    nn_val = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    bitlen = random.randint(1, getbitlen(nn_val))
    cnt = random.randint(0, bitlen-1)

    res = []

    if (op == "NN_ROTATE_LEFT"):  # NN_ROTATE_LEFT
        nn_exp_res = ((nn_val << cnt) ^ (nn_val >> (bitlen - cnt))) & (2**bitlen - 1)
    else: 			  # NN_ROTATE_RIGHT
        nn_exp_res = ((nn_val >> cnt) ^ (nn_val << (bitlen - cnt))) & (2**bitlen - 1)

    fmt = "%s nnuu %s %s %d %d\n"
    res.append(fmt % (op, format_int_string(nn_exp_res, wlen), format_int_string(nn_val, wlen), cnt, bitlen))

    return res

test_funcs["NN_ROTATE_LEFT"] = test_NN_ROTATE
test_funcs["NN_ROTATE_RIGHT"] = test_NN_ROTATE


# Generate tests for NN_XOR, NN_OR and NN_AND operations.
def test_NN_XOR_OR_AND(op):
    # random value for both input numbers
    nn1_val = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn2_val = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    res = []

    if (op == "NN_XOR"):  # NN_XOR
        nn_exp_res = nn1_val ^ nn2_val
    elif (op == "NN_OR"): # NN_OR
        nn_exp_res = nn1_val | nn2_val
    else:                 # NN_AND
        nn_exp_res = nn1_val & nn2_val

    fmt = "%s nnn %s %s %s\n"
    res.append(fmt % (op, format_int_string(nn1_val, wlen), format_int_string(nn2_val, wlen), format_int_string(nn_exp_res, wlen)))

    return res

test_funcs["NN_XOR"] = test_NN_XOR_OR_AND
test_funcs["NN_OR"]  = test_NN_XOR_OR_AND
test_funcs["NN_AND"]  = test_NN_XOR_OR_AND


def test_NN_NOT(op):
    """ Generate tests for NN_NOT """
    nn_val = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    res = []

    # python sucks at computing logical not. It generates a two's complement
    # representation, which mays result in a negative value i.e. not what we
    # are looking for.
    nn_exp_res = ((2 ** getwlenbitlen(nn_val, wlen)) - 1) & (~nn_val)

    fmt = "%s nn %s %s\n"
    res.append(fmt % (op, format_int_string(nn_val, wlen), format_int_string(nn_exp_res, wlen)))

    return res

test_funcs["NN_NOT"] = test_NN_NOT


def test_NN_ADD_SUB(op):
    """ Generate tests for NN_ADD and NN_SUB """
    # Get two random big num
    nn_val1 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_val2 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    # Compute the result depending on the operation
    if (op == "NN_ADD"):
        res = nn_val1 + nn_val2
    else:
        if (nn_val1 < nn_val2):
            tmp = nn_val1
            nn_val1 = nn_val2
            nn_val2 = tmp
        res = nn_val1 - nn_val2
    nn_exp_res = res

    fmt = "%s nnn %s %s %s\n"
    s = fmt % (op, format_int_string(nn_val1, wlen), format_int_string(nn_val2, wlen), format_int_string(nn_exp_res, wlen))

    return [ s ]

test_funcs["NN_ADD"] = test_NN_ADD_SUB
test_funcs["NN_SUB"] = test_NN_ADD_SUB


def test_NN_INC_DEC(op):
    """ Generate tests for NN_INC and NN_DEC """
    nn_val = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    # Compute the result depending on the operation
    if (op == "NN_INC"):
        res = nn_val + 1
    else:
        res = nn_val - 1
    nn_exp_res = res

    fmt = "%s nn %s %s\n"
    s = fmt % (op, format_int_string(nn_val, wlen), format_int_string(nn_exp_res, wlen))

    return [ s ]

test_funcs["NN_INC"] = test_NN_INC_DEC
test_funcs["NN_DEC"] = test_NN_INC_DEC

def test_NN_MOD_ADD_SUB(op):
    """ Generate tests for modular NN_ADD and NN_SUB """
    # Get three random big num
    nn_mod = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_val1 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_mod
    nn_val2 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_mod

    # Compute the result depending on the operation
    if (op == "NN_MOD_ADD"):
        res = (nn_val1 + nn_val2) % nn_mod
    else:
        if (nn_val1 < nn_val2):
            tmp = nn_val1
            nn_val1 = nn_val2
            nn_val2 = tmp
        res = (nn_val1 - nn_val2) % nn_mod
    nn_exp_res = res

    fmt = "%s nnnn %s %s %s %s\n"
    s = fmt % (op, format_int_string(nn_val1, wlen), format_int_string(nn_val2, wlen), format_int_string(nn_mod, wlen), format_int_string(nn_exp_res, wlen))

    return [ s ]

test_funcs["NN_MOD_ADD"] = test_NN_MOD_ADD_SUB
test_funcs["NN_MOD_SUB"] = test_NN_MOD_ADD_SUB

def test_NN_MOD_INC_DEC(op):
    """ Generate tests for NN_MOD_INC and NN_MOD_DEC """
    nn_mod = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_val = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_mod

    # Compute the result depending on the operation
    if (op == "NN_MOD_INC"):
        res = (nn_val + 1) % nn_mod
    else:
        if nn_val == 0:
            nn_val = nn_val + 1
        res = (nn_val - 1) % nn_mod
    nn_exp_res = res

    fmt = "%s nnn %s %s %s\n"
    s = fmt % (op, format_int_string(nn_val, wlen), format_int_string(nn_mod, wlen), format_int_string(nn_exp_res, wlen))

    return [ s ]

test_funcs["NN_MOD_INC"] = test_NN_MOD_INC_DEC
test_funcs["NN_MOD_DEC"] = test_NN_MOD_INC_DEC


def test_NN_MUL(op):
    """ Generate tests for NN_MUL """
    # random value for input numbers
    nn_in1 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_in2 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    out = nn_in1 * nn_in2

    fmt = "%s nnnu %s %s %s\n"
    s = fmt % (op, format_int_string(out, wlen), format_int_string(nn_in1, wlen), format_int_string(nn_in2, wlen))

    return [ s ]

test_funcs["NN_MUL"] = test_NN_MUL

def test_NN_MOD(op):
    """ Generate tests for NN_MOD """
    # random value for input numbers
    nn_c = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_d = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    nn_exp_r = nn_c % nn_d

    fmt = "%s nnn %s %s %s\n"
    s = fmt % (op, format_int_string(nn_exp_r, wlen), format_int_string(nn_c, wlen), format_int_string(nn_d, wlen))

    return [ s ]

test_funcs["NN_MOD"] = test_NN_MOD

def test_NN_DIVREM(op):
    """ Generate tests for NN_DIVREM """
    # random value for input numbers with second one non zero
    nn_c = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_d = 0
    while nn_d == 0:
        nn_d = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    nn_exp_q = (nn_c // nn_d)
    nn_exp_r = nn_c % nn_d

    fmt = "%s nnnn %s %s %s %s\n"
    s = fmt % (op, format_int_string(nn_exp_q, wlen), format_int_string(nn_exp_r, wlen), format_int_string(nn_c, wlen), format_int_string(nn_d, wlen))

    return [ s ]

test_funcs["NN_DIVREM"] = test_NN_DIVREM

def test_NN_XGCD(op):
    """ Generate tests for NN_XGCD """
    # random value for input numbers
    nn_a = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_b = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    (nn_exp_g, nn_exp_u, nn_exp_v) = egcd(nn_a, nn_b)
    # Check sign of u and v and adapt output
    if nn_exp_u < 0:
        sign = -1
        nn_exp_u = (((2 ** getbitlen(nn_exp_u)) - 1) & (~nn_exp_u)) + 1
    else:
        sign = 1
        nn_exp_v = (((2 ** getbitlen(nn_exp_v)) - 1) & (~nn_exp_v)) + 1
    fmt = "%s nnnnnu %s %s %s %s %s %d\n"
    s = fmt % (op, format_int_string(nn_exp_g, wlen), format_int_string(nn_exp_u, wlen), format_int_string(nn_exp_v, wlen), format_int_string(nn_a, wlen), format_int_string(nn_b, wlen), sign)

    return [ s ]

test_funcs["NN_XGCD"] = test_NN_XGCD

def test_NN_GCD(op):
    """ Generate tests for NN_GCD """
    # random value for input numbers
    nn_a = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_b = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    (nn_exp_g, nn_exp_u, nn_exp_v) = egcd(nn_a, nn_b)

    fmt = "%s nnn %s %s %s\n"
    s = fmt % (op, format_int_string(nn_exp_g, wlen), format_int_string(nn_a, wlen), format_int_string(nn_b, wlen))

    return [ s ]

test_funcs["NN_GCD"] = test_NN_GCD

def test_NN_MODINV(op):
    """ Generate tests for NN_MODINV """
    # random value for input numbers, first one non zero
    nn_x = 0
    while nn_x == 0:
        nn_x = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)
    nn_m = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN)

    try:
        nn_exp_v = modinv(nn_x, nn_m)
        exp_r = 1
    except Exception:
        nn_exp_v = 0
        exp_r = 0

    fmt = "%s nnnu %s %s %s %d\n"
    s = fmt % (op, format_int_string(nn_exp_v, wlen), format_int_string(nn_x, wlen), format_int_string(nn_m, wlen), exp_r)

    return [ s ]

test_funcs["NN_MODINV"] = test_NN_MODINV

def test_NN_MODINV_2EXP(op):
    """ Generate tests for NN_MODINV_2EXP """
    # random value for input number, must be odd
    nn_x = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) | 1
    exp = random.randint(1, MAX_INPUT_PARAM_WLEN * wlen)

    nn_exp_v = modinv(nn_x, 2**exp)

    fmt = "%s nnuu %s %s %d %d\n"
    s = fmt % (op, format_int_string(nn_exp_v, wlen), format_int_string(nn_x, wlen), exp, 1)

    return [ s ]

test_funcs["NN_MODINV_2EXP"] = test_NN_MODINV_2EXP

def test_NN_MUL_REDC1(op):
    """ Generate tests for NN_MUL_REDC1 """
    # Odd modulus
    nn_mod = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) | 1
    nn_r, nn_r_square, mpinv = compute_monty_coef(nn_mod, getwlenbitlen(nn_mod, wlen))

    # random value for input numbers modulo our random mod
    nn_in1 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_mod
    nn_in2 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_mod

    # Montgomery multiplication computes in1 * in2 * r^-1 (mod)
    out = (nn_in1 * nn_in2 * modinv(nn_r, nn_mod)) % nn_mod

    fmt = "%s nnnnu %s %s %s %s %d\n"
    s = fmt % (op, format_int_string(out, wlen), format_int_string(nn_in1, wlen), format_int_string(nn_in2, wlen), format_int_string(nn_mod, wlen), mpinv)

    return [ s ]

test_funcs["NN_MUL_REDC1"] = test_NN_MUL_REDC1

def test_NN_COEF_REDC1(op):
    """ Generate tests for NN_COEF_REDC1 """
    # Odd modulus
    nn_mod = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) | 1
    # Get the results
    # Expand the modulus size if necessary
    if getwlenbitlen(nn_mod, wlen) == wlen:
        nn_r, nn_r_square, mpinv = compute_monty_coef(nn_mod, 2*getwlenbitlen(nn_mod, wlen))
    else:
        nn_r, nn_r_square, mpinv = compute_monty_coef(nn_mod, getwlenbitlen(nn_mod, wlen))

    fmt = "%s nnnu %s %s %s %d\n"
    s = fmt % (op, format_int_string(nn_r, wlen), format_int_string(nn_r_square, wlen), format_int_string(nn_mod, wlen), mpinv)

    return [ s ]

test_funcs["NN_COEF_REDC1"] = test_NN_COEF_REDC1

def test_NN_COEF_DIV(op):
    """ Generate tests for NN_COEF_DIV """
    # Odd modulus
    nn_mod = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) | 1
    # Get the results
    # Expand the modulus size if necessary
    if getwlenbitlen(nn_mod, wlen) == wlen:
        pshift, nn_pnorm, prec = compute_div_coef(nn_mod, 2*getwlenbitlen(nn_mod, wlen))
    else:
        pshift, nn_pnorm, prec = compute_div_coef(nn_mod, getwlenbitlen(nn_mod, wlen))

    fmt = "%s nuun %s %d %d %s\n"
    s = fmt % (op, format_int_string(nn_pnorm, wlen), pshift, prec, format_int_string(nn_mod, wlen))

    return [ s ]

test_funcs["NN_COEF_DIV"] = test_NN_COEF_DIV


# Helper to compute and export an Fp context
def format_fp_context(nn_p, wlen):
    nn_nbits = getwlenbitlen(nn_p, wlen)
    if nn_nbits == wlen:
        nn_nbits = 2 * wlen
    nn_r, nn_r_square, mpinv = compute_monty_coef(nn_p, nn_nbits)
    pshift, nn_pnorm, prec = compute_div_coef(nn_p, nn_nbits)
    f = "%%0%dx" % ((nn_nbits // 8) * 2)
    fmpinv = "%%0%dx" % (((wlen // 8)) * 2)
    return ("%s%s%s%s%s%s%s" % (f % nn_p, f % nn_r, f % nn_r_square, fmpinv % mpinv,
               fmpinv % pshift, f % nn_pnorm, fmpinv % prec))

def test_FP_ADD_SUB(op):
    """ Generate tests for FP_ADD_SUB """
    # Get random prime
    #nn_p = random.randint(0, nn_maxval)
    #while not is_probprime(nn_p):
    #    nn_p = random.randint(0, nn_maxval)
    # Use random odd number for faster generation
    nn_p = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) | 1

    # Get two random big num
    fp_val1 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p
    fp_val2 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p

    # Compute the result depending on the operation
    if (op == "FP_ADD"):
        fp_exp_res = (fp_val1 + fp_val2) % nn_p
    else:
        fp_exp_res = (fp_val1 - fp_val2) % nn_p

    fmt = "%s cfff %s %s %s %s\n"
    s = fmt % (op, format_fp_context(nn_p, wlen), format_int_string(fp_exp_res, wlen), format_int_string(fp_val1, wlen), format_int_string(fp_val2, wlen))

    return [ s ]

test_funcs["FP_ADD"] = test_FP_ADD_SUB
test_funcs["FP_SUB"] = test_FP_ADD_SUB


def test_FP_MUL_DIV(op):
    """ Generate tests for FP_MUL_DIV """
    # Get random prime
    #nn_p = random.randint(0, nn_maxval)
    #while not is_probprime(nn_p):
    #    nn_p = random.randint(0, nn_maxval)
    # Use random odd number for faster generation
    nn_p = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) | 1

    # Get two random big num
    fp_val1 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p
    fp_val2 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p
    # p is not prime, so make sure fp_val2 is invertible
    if (op == "FP_DIV"):
        while egcd(fp_val2, nn_p)[0] != 1:
            fp_val2 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p

    # Compute the result depending on the operation
    if (op == "FP_MUL"):
        fp_exp_res = (fp_val1 * fp_val2) % nn_p
    else:
        fp_exp_res = (fp_val1 * modinv(fp_val2, nn_p)) % nn_p

    fmt = "%s cfff %s %s %s %s\n"
    s = fmt % (op,format_fp_context(nn_p, wlen), format_int_string(fp_exp_res, wlen), format_int_string(fp_val1, wlen), format_int_string(fp_val2, wlen))

    return [ s ]

test_funcs["FP_MUL"] = test_FP_MUL_DIV
test_funcs["FP_DIV"] = test_FP_MUL_DIV


def test_FP_MUL_REDC1(op):
    """ Generate tests for FP_MUL_REDC1 """
    # Get random prime
    #while not is_probprime(nn_p):
    #    nn_p = random.randint(0, nn_maxval)
    # Use random odd number for faster generation
    nn_p = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) | 1
    if getwlenbitlen(nn_p, wlen) == wlen:
        nn_r, nn_r_square, mpinv = compute_monty_coef(nn_p, 2*getwlenbitlen(nn_p, wlen))
    else:
        nn_r, nn_r_square, mpinv = compute_monty_coef(nn_p, getwlenbitlen(nn_p, wlen))

    # Get two random big num
    fp_val1 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p
    fp_val2 = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p
    # representations of fp_val1 and fp_val2 in Montgomery's world
    fp_val1_mont = (fp_val1 * nn_r ) % nn_p
    fp_val2_mont = (fp_val2 * nn_r ) % nn_p

    fp_exp_res = (fp_val1_mont * fp_val2_mont * modinv(nn_r%nn_p, nn_p)) % nn_p

    fmt = "%s cfff %s %s %s %s\n"
    s = fmt % (op, format_fp_context(nn_p, wlen), format_int_string(fp_exp_res, wlen), format_int_string(fp_val1_mont, wlen), format_int_string(fp_val2_mont, wlen))

    return [ s ]

test_funcs["FP_MUL_REDC1"] = test_FP_MUL_REDC1


def test_FP_POW(op):
    """ Generate tests for FP_POW """
    # Instead of random prime, use random odd number for faster generation
    nn_p = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) | 1

    # Get two random big num
    fp_val = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p
    nn_exp = get_random_bigint(wlen, MAX_INPUT_PARAM_WLEN) % nn_p

    fp_exp_res = pow(fp_val, nn_exp, nn_p)

    fmt = "%s cffn %s %s %s %s\n"
    s = fmt % (op, format_fp_context(nn_p, wlen), format_int_string(fp_exp_res, wlen), format_int_string(fp_val, wlen), format_int_string(nn_exp, wlen))

    return [ s ]

test_funcs["FP_POW"] = test_FP_POW


def do_test(sockfd, op, n):
    """
    Do given test and send back result on given socket
    before leaving.
    """
    res = []
    for k in range(n):
        res += test_funcs[op](op)
    remain = "".join(res)
    while (remain):
        cur = remain[:DEFBUFSIZE]
        sockfd.send(cur)
        remain = remain[DEFBUFSIZE:]
    sockfd.close()


# The way we make generation parallel is by splitting ntests equally
# on the number of proc we have.
signal.signal(signal.SIGINT, handler)

# ATM, we spawn as many processes as the number of tests we have.
# We adapt the output on stdout or regular file.
if testfile == "stdout":
    fd = sys.stdout
else:
    fd = open(testfile, "w")
numproc = get_cpu_count()
line = 0

sys.stderr.write("[+] Dispatching our %d tests on %d proc\n" % (ntests, numproc))

for test in asked_tests:
    socks = []

    # Before forking, we need to be sure there is no
    # remaining data in file fd buffers, otherwise
    # those will end up being flushed by out childs
    # and create duplicates in the file.
    fd.flush()

    n = max((ntests // numproc), 1)
    for k in range(0, ntests, n):
        # Create a pair of sockets for us and the child we'll spawn
        a, b = socket.socketpair()
        a.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, DEFBUFSIZE)
        b.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, DEFBUFSIZE)
        a.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, DEFBUFSIZE)
        b.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, DEFBUFSIZE)

        # keep track of our socket
        socks.append(a)

        # Double fork a helper to create the campaign for this specific
        # test.
        pid = os.fork()
        if pid != 0:
            os.waitpid(pid, 0)
        else:
            pid = os.fork()
            if pid != 0:
                sys.exit()
            else:
                a.close()
                do_test(b, test, n)
                sys.exit()

        b.close()

    # Now that we have fork all helpers, let's just wait until they are
    # done and read back the results to write them to file.
    while socks:
        tmp = select.select(socks, [], [])
        for s in tmp[0]:
            socks.remove(s)
            tmp = s.recv(DEFBUFSIZE)
            res = ""
            while tmp:
                res += tmp
                tmp = s.recv(DEFBUFSIZE)
            res = res.split('\n')[:-1]
            reslen = len(res)
            res = zip(range(reslen), res)
            fd.write("".join(map(lambda (x,y): "%d %s\n" % (x + line, y), res)))
            line += reslen

fd.close()



#!/usr/bin/env python3

"""
MPFS HSS Deflate tool

This script takes an HSS-L2LIM image and compresses it for storage in eNVM
using Deflate

"""

#
#
# MPFS HSS Deflate tool
#
# Copyright 2021-2022 Microchip Corporation.
#
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
#
#

import argparse
import sys
try:
    import zlib
except:
    print("Unable to import the zlib package, please check your Python installation")
    print("If you are attempting to build the HSS using SoftConsole on Linux there is a known issue when importing the zlib module")
    print("See: https://github.com/orgs/polarfire-soc/discussions/276 for more information")
    sys.exit(1)


def deflate(data: bytes, compression_level: int = 9) -> bytes:
    '''Takes bytes and a compression level, returns compressed bytes'''
    compressor = zlib.compressobj(level=compression_level,
                                  method=zlib.DEFLATED,
                                  wbits=zlib.MAX_WBITS,
                                  memLevel=zlib.DEF_MEM_LEVEL,
                                  strategy=0)
    deflated_bytes = compressor.compress(data) + compressor.flush()
    return deflated_bytes


def inflate(data: bytes) -> bytes:
    '''Takes compressed bytes and a compression level,
    returns decompressed bytes'''
    decompressor = zlib.decompressobj(-zlib.MAX_WBITS)
    inflated_bytes = decompressor.decompress(data) + decompressor.flush()
    return inflated_bytes


def get_script_version() -> str:
    '''returns script version'''
    return "1.0.1"


def get_header_bytes(compressed_crc: int, original_crc: int,
                     compressed_image_len: int,
                     original_image_len: int) -> bytearray:
    '''takes compressed and original CRCs and lengths,
    returns the payload header'''
    #
    # This function must output a header that is in-sync with
    # include/hss-types.h ...
    #
    # #pragma pack(8)
    # struct HSS_CompressedImage {
    #     uint32_t magic;
    #     uint32_t version;
    #     size_t headerLength;
    #     uint32_t headerCrc;
    #     uint32_t compressedCrc;
    #     uint32_t originalCrc;
    #     size_t compressedImageLen;
    #     size_t originalImageLen;
    #     uint8_t hash[32];
    #     uint8_t ecdsaSig[32];
    # };
    # #define mHSS_COMPRESSED_MAGIC    (0xC08B8355u)
    # #define mHSS_COMPRESSED_VERSION_FASTLZ 1u
    # #define mHSS_COMPRESSED_VERSION_MINIZ  2u
    #
    # offsetof(magic):               0
    # offsetof(version):             4
    # offsetof(headerLength):        8
    # offsetof(headerCrc):           16
    # offsetof(compressedCrc):       20
    # offsetof(originalCrc):         24
    # offsetof(compressedImageLen):  32
    # offset_of(originalImageLen):   40
    # offsetof(hash):                48
    # offsetof(ecdsaSig):            80

    header = bytearray(112)
    header[0:3] = 0xC08B8355.to_bytes(4, "little")  # magic
    header[4:8] = 0x2.to_bytes(4, "little")         # version
    header[8:16] = 0x0.to_bytes(8, "little")        # headerLength placeholder
    header[16:20] = 0x0.to_bytes(4, "little")       # headerCrc placeholder
    header[20:24] = compressed_crc.to_bytes(4, "little")
    header[24:32] = original_crc.to_bytes(4, "little")
    header[32:40] = compressed_image_len.to_bytes(8, "little")
    header[40:48] = original_image_len.to_bytes(8, "little")

    header[48:80] = bytearray(32)                   # hash32 placeholder
    header[80:112] = bytearray(32)                  # ecdsaSig32 placeholder

    header_length = len(header)
    header[8:12] = header_length.to_bytes(4, "little")
    header[12:16] = 0x0.to_bytes(4, "little")       # padding

    header_crc = zlib.crc32(header)
    header[16:20] = header_crc.to_bytes(4, "little")

    global args
    if args.verbose:
        print("Header length is %d (0x%08X)" % (header_length, header_length))
        print("Header CRC is 0x%08X" % (header_crc))

    return header


def main():
    '''main function'''
    parser = argparse.ArgumentParser(description='Deflate HSS-L2LIM image')
    parser.add_argument('--verbose', '-v', action='count', default=0)
    parser.add_argument('imageFileIn')
    parser.add_argument('deflatedFileOut')

    global args
    args = parser.parse_args()

    if args.verbose:
        print("Reading image from " + args.imageFileIn)
    with open(args.imageFileIn, 'rb') as file_in:
        image_data = file_in.read()

    image_len = len(image_data)
    if args.verbose:
        print("Reading %d (0x%08X) bytes" % (image_len, image_len))

    image_crc = zlib.crc32(image_data)

    if args.verbose:
        print("Image CRC is 0x%08X" % (image_crc))
        print("Deflating...")

    deflated_data = deflate(image_data)
    deflated_len = len(deflated_data)

    deflated_crc = zlib.crc32(deflated_data)

    if args.verbose:
        print("Deflated CRC is 0x%08X" % (deflated_crc))
        print("Deflated length %d (0x%08X) bytes"
              % (deflated_len, deflated_len))
        print("Outputting to " + args.deflatedFileOut)

    header_data = get_header_bytes(compressed_crc=deflated_crc,
                                   original_crc=image_crc,
                                   compressed_image_len=deflated_len,
                                   original_image_len=image_len)
    with open(args.deflatedFileOut, "wb") as file_out:
        output_size = file_out.write(header_data)
        output_size += file_out.write(deflated_data)
    if args.verbose:
        print("Wrote %d (0x%08X) bytes" % (output_size, output_size))

#
#
#


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright (c) 2018-2019, Ulf Magnusson
# SPDX-License-Identifier: ISC

"""
Writes a configuration file where all symbols are set to their their default
values.

The default output filename is '.config'. A different filename can be passed in
the KCONFIG_CONFIG environment variable.

Usage for the Linux kernel:

  $ make [ARCH=<arch>] scriptconfig SCRIPT=Kconfiglib/alldefconfig.py
"""
import kconfiglib


def main():
    kconf = kconfiglib.standard_kconfig(__doc__)
    kconf.load_allconfig("alldef.config")
    print(kconf.write_config())


if __name__ == "__main__":
    main()



# Prints the menu tree of the configuration. Dependencies between symbols can
# sometimes implicitly alter the menu structure (see kconfig-language.txt), and
# that's implemented too.
#
# Note: See the Kconfig.node_iter() function as well, which provides a simpler
# interface for walking the menu tree.
#
# Usage:
#
#   $ make [ARCH=<arch>] scriptconfig SCRIPT=Kconfiglib/examples/print_tree.py
#
# Example output:
#
#   ...
#   config HAVE_KERNEL_LZO
#   config HAVE_KERNEL_LZ4
#   choice
#     config KERNEL_GZIP
#     config KERNEL_BZIP2
#     config KERNEL_LZMA
#     config KERNEL_XZ
#     config KERNEL_LZO
#     config KERNEL_LZ4
#   config DEFAULT_HOSTNAME
#   config SWAP
#   config SYSVIPC
#     config SYSVIPC_SYSCTL
#   config POSIX_MQUEUE
#     config POSIX_MQUEUE_SYSCTL
#   config CROSS_MEMORY_ATTACH
#   config FHANDLE
#   config USELIB
#   config AUDIT
#   config HAVE_ARCH_AUDITSYSCALL
#   config AUDITSYSCALL
#   config AUDIT_WATCH
#   config AUDIT_TREE
#   menu "IRQ subsystem"
#     config MAY_HAVE_SPARSE_IRQ
#     config GENERIC_IRQ_LEGACY
#     config GENERIC_IRQ_PROBE
#   ...

import sys

from kconfiglib import Kconfig, Symbol, Choice, MENU, COMMENT


def indent_print(s, indent):
    print(indent*" " + s)


def print_items(node, indent):
    while node:
        if isinstance(node.item, Symbol):
            indent_print("config " + node.item.name, indent)

        elif isinstance(node.item, Choice):
            indent_print("choice", indent)

        elif node.item == MENU:
            indent_print('menu "{}"'.format(node.prompt[0]), indent)

        elif node.item == COMMENT:
            indent_print('comment "{}"'.format(node.prompt[0]), indent)


        if node.list:
            print_items(node.list, indent + 2)

        node = node.next


kconf = Kconfig(sys.argv[1])
print_items(kconf.top_node, 0)



#!/usr/bin/env python3

# Copyright (c) 2018-2019, Ulf Magnusson
# SPDX-License-Identifier: ISC

"""
Updates an old .config file or creates a new one, by filling in default values
for all new symbols. This is the same as picking the default selection for all
symbols in oldconfig, or entering the menuconfig interface and immediately
saving.

The default input/output filename is '.config'. A different filename can be
passed in the KCONFIG_CONFIG environment variable.

When overwriting a configuration file, the old version is saved to
<filename>.old (e.g. .config.old).
"""
import kconfiglib


def main():
    kconf = kconfiglib.standard_kconfig(__doc__)
    print(kconf.load_config())
    print(kconf.write_config())


if __name__ == "__main__":
    main()



# Does a case-insensitive search for a regular expression in the help texts of
# symbols and choices and the prompts of menus and comments. Prints the
# matching items together with their locations and the matching text.
#
# Usage:
#
#   $ make [ARCH=<arch>] scriptconfig SCRIPT=Kconfiglib/examples/help_grep.py SCRIPT_ARG=<regex>
#
# Shortened example output for SCRIPT_ARG=general:
#
#   menu "General setup"
#   location: init/Kconfig:39
#
#   config SYSVIPC
#   	bool
#   	prompt "System V IPC"
#   	help
#   	  ...
#   	  exchange information. It is generally considered to be a good thing,
#   	  ...
#
#   location: init/Kconfig:233
#
#   config BSD_PROCESS_ACCT
#   	bool
#   	prompt "BSD Process Accounting" if MULTIUSER
#   	help
#   	  ...
#   	  information.  This is generally a good idea, so say Y.
#
#   location: init/Kconfig:403
#
#   ...


import re
import sys

from kconfiglib import Kconfig, Symbol, Choice, MENU, COMMENT


if len(sys.argv) < 3:
    sys.exit("Pass the regex with SCRIPT_ARG=<regex>")

search = re.compile(sys.argv[2], re.IGNORECASE).search

for node in Kconfig(sys.argv[1]).node_iter():
    match = False

    if isinstance(node.item, (Symbol, Choice)) and \
       node.help is not None and search(node.help):
        print(node.item)
        match = True

    elif node.item == MENU and search(node.prompt[0]):
        print('menu "{}"'.format(node.prompt[0]))
        match = True

    elif node.item == COMMENT and search(node.prompt[0]):
        print('comment "{}"'.format(node.prompt[0]))
        match = True

    if match:
        print("location: {}:{}\n".format(node.filename, node.linenr))



#!/usr/bin/env python3

# This script functions similarly to scripts/kconfig/merge_config.sh from the
# kernel tree, merging multiple configurations fragments to produce a complete
# .config, with unspecified values filled in as for alldefconfig.
#
# The generated .config respects symbol dependencies, and a warning is printed
# if any symbol gets a different value from the assigned value.
#
# For a real-world merging example based on this script, see
# https://github.com/zephyrproject-rtos/zephyr/blob/master/scripts/kconfig/kconfig.py.
#
# Here's a demo:
#
# Kconfig contents:
#
#     config FOO
#         bool "FOO"
#
#     config BAR
#         bool "BAR"
#
#     config BAZ
#         string "BAZ"
#
#     config QAZ
#         bool "QAZ" if n
#
#
# conf1 contents:
#
#     CONFIG_FOO=y
#
#
# conf2 contents:
#
#     CONFIG_BAR=y
#
#
# conf3 contents:
#
#     # Assigned twice (would generate warning if 'warn_assign_override' was
#     # True)
#     # CONFIG_FOO is not set
#
#     # Ops... this symbol doesn't exist
#     CONFIG_OPS=y
#
#     CONFIG_BAZ="baz string"
#
#
# conf4 contents:
#
#     CONFIG_QAZ=y
#
#
# Running:
#
#     $ python(3) merge_config.py Kconfig merged conf1 conf2 conf3 conf4
#     Merged configuration 'conf1'
#     Merged configuration 'conf2'
#     conf3:5: warning: attempt to assign the value 'y' to the undefined symbol OPS
#     Merged configuration 'conf3'
#     Merged configuration 'conf4'
#     Configuration saved to 'merged'
#     warning: QAZ (defined at Kconfig:10) was assigned the value 'y' but got the value 'n' -- check dependencies
#     $ cat merged
#     Generated by Kconfiglib (https://github.com/ulfalizer/Kconfiglib)
#     # CONFIG_FOO is not set
#     CONFIG_BAR=y
#     CONFIG_BAZ="baz string"

from __future__ import print_function
import sys

from kconfiglib import Kconfig, BOOL, TRISTATE, TRI_TO_STR


if len(sys.argv) < 4:
    sys.exit("usage: merge_config.py Kconfig merged_config config1 [config2 ...]")

kconf = Kconfig(sys.argv[1], suppress_traceback=True)

# Enable warnings for assignments to undefined symbols
kconf.warn_assign_undef = True

# (This script uses alldefconfig as the base. Other starting states could be
# set up here as well. The approach in examples/allnoconfig_simpler.py could
# provide an allnoconfig starting state for example.)

# Disable warnings generated for multiple assignments to the same symbol within
# a (set of) configuration files. Assigning a symbol multiple times might be
# done intentionally when merging configuration files.
kconf.warn_assign_override = False
kconf.warn_assign_redun = False

# Create a merged configuration by loading the fragments with replace=False.
# load_config() and write_config() returns a message to print.
for config in sys.argv[3:]:
    print(kconf.load_config(config, replace=False))

# Write the merged configuration
print(kconf.write_config(sys.argv[2]))

# Print warnings for symbols whose actual value doesn't match the assigned
# value
for sym in kconf.defined_syms:
    # Was the symbol assigned to?
    if sym.user_value is not None:
        # Tristate values are represented as 0, 1, 2. Having them as
        # "n", "m", "y" is more convenient here, so convert.
        if sym.type in (BOOL, TRISTATE):
            user_value = TRI_TO_STR[sym.user_value]
        else:
            user_value = sym.user_value

        if user_value != sym.str_value:
            print("warning: {} was assigned the value '{}' but got the "
                  "value '{}' -- check dependencies".format(
                      sym.name_and_loc, user_value, sym.str_value),
                  file=sys.stderr)



#!/usr/bin/env python3

# Implements a simple configuration interface on top of Kconfiglib to
# demonstrate concepts for building a menuconfig-like. Emulates how the
# standard menuconfig prints menu entries.
#
# Always displays the entire Kconfig tree to keep things as simple as possible
# (all symbols, choices, menus, and comments).
#
# Usage:
#
#   $ python(3) Kconfiglib/examples/menuconfig.py <Kconfig file>
#
# A sample Kconfig is available in Kconfiglib/examples/Kmenuconfig.
#
# Here's a notation guide. The notation matches the one used by menuconfig
# (scripts/kconfig/mconf):
#
#   [ ] prompt      - Bool
#   < > prompt      - Tristate
#   {M} prompt      - Tristate selected to m. Can only be set to m or y.
#   -*- prompt      - Bool/tristate selected to y, pinning it
#   -M- prompt      - Tristate selected to m that also has m visibility,
#                     pinning it to m
#   (foo) prompt    - String/int/hex symbol with value "foo"
#   --> prompt      - The selected symbol in a choice in y mode. This
#                     syntax is unique to this example.
#
# When modules are disabled, the .type attribute of TRISTATE symbols and
# choices automatically changes to BOOL. This trick is used by the C
# implementation as well, and gives the expected behavior without having to do
# anything extra here. The original type is available in .orig_type if needed.
#
# The Kconfiglib/examples/Kmenuconfig example uses named choices to be able to
# refer to choices by name. Named choices are supported in the C tools too, but
# I don't think I've ever seen them used in the wild.
#
# Sample session:
#
#   $ python Kconfiglib/examples/menuconfig.py Kconfiglib/examples/Kmenuconfig
#
#   ======== Example Kconfig configuration ========
#
#   [*] Enable loadable module support (MODULES)
#       Bool and tristate symbols
#           [*] Bool symbol (BOOL)
#                   [ ] Dependent bool symbol (BOOL_DEP)
#                   < > Dependent tristate symbol (TRI_DEP)
#                   [ ] First prompt (TWO_MENU_NODES)
#           < > Tristate symbol (TRI)
#           [ ] Second prompt (TWO_MENU_NODES)
#               *** These are selected by TRI_DEP ***
#           < > Tristate selected by TRI_DEP (SELECTED_BY_TRI_DEP)
#           < > Tristate implied by TRI_DEP (IMPLIED_BY_TRI_DEP)
#       String, int, and hex symbols
#           (foo) String symbol (STRING)
#           (747) Int symbol (INT)
#           (0xABC) Hex symbol (HEX)
#       Various choices
#           -*- Bool choice (BOOL_CHOICE)
#                   --> Bool choice sym 1 (BOOL_CHOICE_SYM_1)
#                       Bool choice sym 2 (BOOL_CHOICE_SYM_2)
#           {M} Tristate choice (TRI_CHOICE)
#                   < > Tristate choice sym 1 (TRI_CHOICE_SYM_1)
#                   < > Tristate choice sym 2 (TRI_CHOICE_SYM_2)
#           [ ] Optional bool choice (OPT_BOOL_CHOICE)
#
#   Enter a symbol/choice name, "load_config", or "write_config" (or press CTRL+D to exit): BOOL
#   Value for BOOL (available: n, y): n
#
#   ======== Example Kconfig configuration ========
#
#   [*] Enable loadable module support (MODULES)
#       Bool and tristate symbols
#           [ ] Bool symbol (BOOL)
#           < > Tristate symbol (TRI)
#           [ ] Second prompt (TWO_MENU_NODES)
#               *** These are selected by TRI_DEP ***
#           < > Tristate selected by TRI_DEP (SELECTED_BY_TRI_DEP)
#           < > Tristate implied by TRI_DEP (IMPLIED_BY_TRI_DEP)
#       String, int, and hex symbols
#           (foo) String symbol (STRING)
#           (747) Int symbol (INT)
#           (0xABC) Hex symbol (HEX)
#       Various choices
#           -*- Bool choice (BOOL_CHOICE)
#                   --> Bool choice sym 1 (BOOL_CHOICE_SYM_1)
#                       Bool choice sym 2 (BOOL_CHOICE_SYM_2)
#           {M} Tristate choice (TRI_CHOICE)
#                   < > Tristate choice sym 1 (TRI_CHOICE_SYM_1)
#                   < > Tristate choice sym 2 (TRI_CHOICE_SYM_2)
#          [ ] Optional bool choice (OPT_BOOL_CHOICE)
#
#   Enter a symbol/choice name, "load_config", or "write_config" (or press CTRL+D to exit): MODULES
#   Value for MODULES (available: n, y): n
#
#   ======== Example Kconfig configuration ========
#
#   [ ] Enable loadable module support (MODULES)
#       Bool and tristate symbols
#           [ ] Bool symbol (BOOL)
#           [ ] Tristate symbol (TRI)
#           [ ] Second prompt (TWO_MENU_NODES)
#               *** These are selected by TRI_DEP ***
#           [ ] Tristate selected by TRI_DEP (SELECTED_BY_TRI_DEP)
#           [ ] Tristate implied by TRI_DEP (IMPLIED_BY_TRI_DEP)
#       String, int, and hex symbols
#           (foo) String symbol (STRING)
#           (747) Int symbol (INT)
#           (0xABC) Hex symbol (HEX)
#       Various choices
#           -*- Bool choice (BOOL_CHOICE)
#                   --> Bool choice sym 1 (BOOL_CHOICE_SYM_1)
#                       Bool choice sym 2 (BOOL_CHOICE_SYM_2)
#           -*- Tristate choice (TRI_CHOICE)
#                   --> Tristate choice sym 1 (TRI_CHOICE_SYM_1)
#                       Tristate choice sym 2 (TRI_CHOICE_SYM_2)
#           [ ] Optional bool choice (OPT_BOOL_CHOICE)
#
#   Enter a symbol/choice name, "load_config", or "write_config" (or press CTRL+D to exit): ^D

from __future__ import print_function
import readline
import sys

from kconfiglib import Kconfig, \
                       Symbol, MENU, COMMENT, \
                       BOOL, TRISTATE, STRING, INT, HEX, UNKNOWN, \
                       expr_value, \
                       TRI_TO_STR


# Python 2/3 compatibility hack
if sys.version_info[0] < 3:
    input = raw_input


def indent_print(s, indent):
    print(indent*" " + s)


def value_str(sc):
    """
    Returns the value part ("[*]", "<M>", "(foo)" etc.) of a menu entry.

    sc: Symbol or Choice.
    """
    if sc.type in (STRING, INT, HEX):
        return "({})".format(sc.str_value)

    # BOOL or TRISTATE

    # The choice mode is an upper bound on the visibility of choice symbols, so
    # we can check the choice symbols' own visibility to see if the choice is
    # in y mode
    if isinstance(sc, Symbol) and sc.choice and sc.visibility == 2:
        # For choices in y mode, print '-->' next to the selected symbol
        return "-->" if sc.choice.selection is sc else "   "

    tri_val_str = (" ", "M", "*")[sc.tri_value]

    if len(sc.assignable) == 1:
        # Pinned to a single value
        return "-{}-".format(tri_val_str)

    if sc.type == BOOL:
        return "[{}]".format(tri_val_str)

    if sc.type == TRISTATE:
        if sc.assignable == (1, 2):
            # m and y available
            return "{" + tri_val_str + "}"  # Gets a bit confusing with .format()
        return "<{}>".format(tri_val_str)


def node_str(node):
    """
    Returns the complete menu entry text for a menu node, or "" for invisible
    menu nodes. Invisible menu nodes are those that lack a prompt or that do
    not have a satisfied prompt condition.

    Example return value: "[*] Bool symbol (BOOL)"

    The symbol name is printed in parentheses to the right of the prompt. This
    is so that symbols can easily be referred to in the configuration
    interface.
    """
    if not node.prompt:
        return ""

    # Even for menu nodes for symbols and choices, it's wrong to check
    # Symbol.visibility / Choice.visibility here. The reason is that a symbol
    # (and a choice, in theory) can be defined in multiple locations, giving it
    # multiple menu nodes, which do not necessarily all have the same prompt
    # visibility. Symbol.visibility / Choice.visibility is calculated as the OR
    # of the visibility of all the prompts.
    prompt, prompt_cond = node.prompt
    if not expr_value(prompt_cond):
        return ""

    if node.item == MENU:
        return "    " + prompt

    if node.item == COMMENT:
        return "    *** {} ***".format(prompt)

    # Symbol or Choice

    sc = node.item

    if sc.type == UNKNOWN:
        # Skip symbols defined without a type (these are obscure and generate
        # a warning)
        return ""

    # {:3} sets the field width to three. Gives nice alignment for empty string
    # values.
    res = "{:3} {}".format(value_str(sc), prompt)

    # Don't print the name for unnamed choices (the normal kind)
    if sc.name is not None:
        res += " ({})".format(sc.name)

    return res


def print_menuconfig_nodes(node, indent):
    """
    Prints a tree with all the menu entries rooted at 'node'. Child menu
    entries are indented.
    """
    while node:
        string = node_str(node)
        if string:
            indent_print(string, indent)

        if node.list:
            print_menuconfig_nodes(node.list, indent + 8)

        node = node.next


def print_menuconfig(kconf):
    """
    Prints all menu entries for the configuration.
    """
    # Print the expanded mainmenu text at the top. This is the same as
    # kconf.top_node.prompt[0], but with variable references expanded.
    print("\n======== {} ========\n".format(kconf.mainmenu_text))

    print_menuconfig_nodes(kconf.top_node.list, 0)
    print("")


def get_value_from_user(sc):
    """
    Prompts the user for a value for the symbol or choice 'sc'. For
    bool/tristate symbols and choices, provides a list of all the assignable
    values.
    """
    if not sc.visibility:
        print(sc.name + " is not currently visible")
        return False

    prompt = "Value for {}".format(sc.name)
    if sc.type in (BOOL, TRISTATE):
        prompt += " (available: {})" \
                  .format(", ".join(TRI_TO_STR[val] for val in sc.assignable))
    prompt += ": "

    val = input(prompt)

    # Automatically add a "0x" prefix for hex symbols, like the menuconfig
    # interface does. This isn't done when loading .config files, hence why
    # set_value() doesn't do it automatically.
    if sc.type == HEX and not val.startswith(("0x", "0X")):
        val = "0x" + val

    # Let Kconfiglib itself print a warning here if the value is invalid. We
    # could also disable warnings temporarily with 'kconf.warn = False' and
    # print our own warning.
    return sc.set_value(val)


if __name__ == "__main__":
    if len(sys.argv) != 2:
        sys.exit("usage: menuconfig.py <Kconfig file>")

    # Load Kconfig configuration files
    kconf = Kconfig(sys.argv[1])

    # Print the initial configuration tree
    print_menuconfig(kconf)

    while True:
        try:
            cmd = input('Enter a symbol/choice name, "load_config", or '
                        '"write_config" (or press CTRL+D to exit): ').strip()
        except EOFError:
            print("")
            break

        if cmd == "load_config":
            config_filename = input(".config file to load: ")
            try:
                # Returns a message telling which file got loaded
                print(kconf.load_config(config_filename))
            except EnvironmentError as e:
                print(e, file=sys.stderr)

            print_menuconfig(kconf)
            continue

        if cmd == "write_config":
            config_filename = input("To this file: ")
            try:
                # Returns a message telling which file got saved
                print(kconf.write_config(config_filename))
            except EnvironmentError as e:
                print(e, file=sys.stderr)

            continue

        # Assume 'cmd' is the name of a symbol or choice if it isn't one of the
        # commands above, prompt the user for a value for it, and print the new
        # configuration tree

        if cmd in kconf.syms:
            if get_value_from_user(kconf.syms[cmd]):
                print_menuconfig(kconf)

            continue

        if cmd in kconf.named_choices:
            if get_value_from_user(kconf.named_choices[cmd]):
                print_menuconfig(kconf)

            continue

        print("No symbol/choice named '{}' in the configuration".format(cmd),
              file=sys.stderr)



# This is tree-walking version of allnoconfig.py, for demonstration purposes.
# Verified by the test suite to generate identical output to 'make allnoconfig'
# for all ARCHes.
#
# Note: A more practical version would use Kconfig.node_iter(). The manual tree
# walking is for demonstration purposes.
#
# Usage for the Linux kernel:
#
#   $ make [ARCH=<arch>] scriptconfig SCRIPT=Kconfiglib/examples/allnoconfig_walk.py

import sys

from kconfiglib import Kconfig, Symbol


def do_allnoconfig(node):
    global changed

    # Walk the tree of menu nodes. You can imagine this as going down/into menu
    # entries in the menuconfig interface, setting each to n (or the lowest
    # assignable value).

    while node:
        if isinstance(node.item, Symbol):
            sym = node.item

            # Is the symbol a non-allnoconfig_y symbol that can be set to a
            # lower value than its current value?
            if (not sym.is_allnoconfig_y and
                sym.assignable and
                sym.assignable[0] < sym.tri_value):

                # Yup, lower it
                sym.set_value(sym.assignable[0])
                changed = True

        # Recursively lower children
        if node.list:
            do_allnoconfig(node.list)

        node = node.next


# Parse the Kconfig files
kconf = Kconfig(sys.argv[1])

# Do an initial pass to set 'option allnoconfig_y' symbols to y
for sym in kconf.unique_defined_syms:
    if sym.is_allnoconfig_y:
        sym.set_value(2)

while True:
    # Changing later symbols in the configuration can sometimes allow earlier
    # symbols to be lowered, e.g. if a later symbol 'select's an earlier
    # symbol. To handle such situations, we do additional passes over the tree
    # until we're no longer able to change the value of any symbol in a pass.
    changed = False

    do_allnoconfig(kconf.top_node)

    # Did the pass change any symbols?
    if not changed:
        break

print(kconf.write_config())
